/*
 * Copyright (c) 1995, 1996, 1997 The University of Utah and
 * the Computer Systems Laboratory at the University of Utah (CSL).
 *
 * This file is part of Flick, the Flexible IDL Compiler Kit.
 *
 * Flick is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Flick is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Flick; see the file COPYING.  If not, write to
 * the Free Software Foundation, 59 Temple Place #330, Boston, MA 02111, USA.
 */

#include <assert.h>
#include <mom/idl_id.h>
#include <mom/mint_special.h>
#include <mom/compiler.h>
#include <mom/libmint.h>
#include <mom/c/libcast.h>
#include <mom/c/libpres_c.h>

#include "mach3.h"

#define m(n) (&pres->mint.defs.defs_val[n])
#define MAX_GLOB_SIZE 1050000
mach3_mu_state::mach3_mu_state(pres_c_1 *pres, mu_state_op op, int assumptions,
			       char *which)
	: mem_mu_state(pres, op, assumptions, 2, 0, MAX_GLOB_SIZE, which)
{
	mach3_array = 0;
	is_complex = 0;
	id_expected = 1;
	is_mig_message = 0;
	is_fixed_string = 0;
	
	/* Set our PRES_C_INLINE_TYPED flag and CAST expression to null. */
	marshaling_inline_typed = 0;
	tag_cexpr = 0;
}

mach3_mu_state::mach3_mu_state(const mach3_mu_state &must)
	: mem_mu_state(must)
{
	mach3_array = must.mach3_array;
	is_complex = must.is_complex;
	id_expected = must.id_expected;
	is_mig_message = must.is_mig_message;
	is_fixed_string = must.is_fixed_string;
	marshaling_inline_typed = must.marshaling_inline_typed;
	tag_cexpr = must.tag_cexpr;
}

mu_state *mach3_mu_state::another(mu_state_op op)
{
	return new mach3_mu_state(pres, op, assumptions, which_stub);
}

mu_state *mach3_mu_state::clone()
{
	return new mach3_mu_state(*this);
}

#define MACH_NOTIFY_FIRST               0100

void mach3_mu_state::mu_mapping_simple(cast_expr expr,
				       cast_type ctype,
				       mint_ref itype)
{
	cast_expr id_expr;
	int id_is_int = ((m(itype))->kind == MINT_INTEGER);
	
	if ((id_expected == 1)
	    && id_is_int) {
		/*
		 * When `id_expected == 1', we're expecting an integer to
		 * identify the IDL in use.
		 */
		id_expr = cast_new_expr_name("_stream->msg_buf->Head.msgh_id");
		
		if (op & MUST_ENCODE) {
			add_stmt(cast_new_stmt_expr(
				cast_new_expr_assign(id_expr, expr)));
			add_stmt(cast_new_if(
				cast_new_binary_expr(
					CAST_BINARY_NE,
					expr,
					cast_new_expr_lit_int(IDL_MIG, 0)),
				cast_new_stmt_expr(
					cast_new_expr_assign(
						id_expr,
						expr)),
				0));
		} else if (op & MUST_DECODE) {
			add_stmt(cast_new_stmt_expr(
				cast_new_expr_assign(
					expr,
					cast_new_expr_cond(
						cast_new_binary_expr(
							CAST_BINARY_LT,
							id_expr,
							cast_new_expr_lit_int(
								MACH_NOTIFY_FIRST,
								0)),
						id_expr,
						cast_new_expr_lit_int(IDL_MIG,
								      0)))));
		}
		/*
		 * Leave id_expected == 1 for our `mu_union_case' override to
		 * see.
		 */
		
	} else if ((id_expected == 2)
		   && id_is_int) {
		/*
		 * When `id_expected == 2', we're expecting an operation
		 * identifier in the Mach message header.
		 */
		id_expr = cast_new_expr_name("_stream->msg_buf->Head.msgh_id");
		
		if (op & MUST_ENCODE) {
			add_stmt(cast_new_stmt_expr(
				cast_new_expr_assign(id_expr, expr)));
		} else if (op & MUST_DECODE) {
			add_stmt(cast_new_stmt_expr(
				cast_new_expr_assign(expr, id_expr))); 
		}
		
		/*
		 * If we are processing a reply, the next thing to be handled
		 * will be the operation reply code.  When we are processing a
		 * request, nothing special is coming up.
		 */
		if (   ((op & MUST_DECODE) && !strcmp(which_stub, "client"))
		    || ((op & MUST_ENCODE) && !strcmp(which_stub, "server"))
		    )
			id_expected = 3;
		else
			id_expected = 0;
		
	} else if ((id_expected == 3)
		   && id_is_int) {
		/*
		 * When `id_expected == 3', we're expecting an operation reply
		 * code (i.e., the discriminator of the reply union, indicating
		 * the operation's success or failure) in the Mach message
		 * header.
		 */
		id_expr = cast_new_expr_name("_stream->msg_buf->RetCode");
		
		if (op & MUST_ENCODE) {
			add_stmt(cast_new_stmt_expr(
				cast_new_expr_assign(id_expr, expr)));
		} else if (op & MUST_DECODE) {
			add_stmt(cast_new_stmt_expr(
				cast_new_expr_assign(expr, id_expr))); 
		}
		
		/* Nothing else special coming now. */
		id_expected = 0;
		
	} else {
		if (id_expected && !id_is_int) {
			warn("Could not store non-integer identifier in Mach "
			     "message field.");
			/* No more special message ID processing. */
			id_expected = 0;
		}
		mem_mu_state::mu_mapping_simple(expr, ctype, itype);
	}
}

void mach3_mu_state::mu_union_case(functor *f, mint_const discrim_val)
{
	int old_id_expected = id_expected;
	
	if (id_expected == 1) {
		assert(discrim_val->kind == MINT_CONST_INT);
		/*
		 * XXX --- Now that we have symbolic constants we should use
		 * them!  But as a stopgap, assert that we have a literal.
		 */
		assert(discrim_val->mint_const_u_u.const_int.kind
		       == MINT_CONST_LITERAL);
		is_mig_message = ((discrim_val->
				   mint_const_u_u.const_int.mint_const_int_u_u.
				   value)
				  == IDL_MIG);
		id_expected = is_mig_message ? 2 : 0;
	}
	
	mem_mu_state::mu_union_case(f, discrim_val);
	
	id_expected = old_id_expected;
}

void mach3_mu_state::mu_mapping_type_tag(cast_expr expr, cast_type /*ctype*/, mint_ref /*itype*/)
{
	if (marshaling_inline_typed)
	{
		/* if descending the type tag branch of an inline_typed,
		   marshaling_inline_typed is true, so mu_mapping_type_tag
		   should simply store the tag's cast_expr into tag_cexpr
		   for later use instead of explicitly [un]marshaling. */
		tag_cexpr = expr;
	} else {
		/* Find the marshaling parameters for this port type.  */
		char *macro_name = flick_asprintf(
			"flick_%s_%s_type_tag", get_be_name(), get_buf_name());
		cast_expr macro_expr = cast_new_expr_name(macro_name);
		cast_expr ofs_expr
			= cast_new_expr_lit_int(chunk_prim(2, 4), 0);
		
		/* Spit out the marshal/unmarshal macro call.  */
		cast_expr cex = cast_new_expr_call(macro_expr, 2);
		cex->cast_expr_u_u.call.params.params_val[0] = ofs_expr;
		cex->cast_expr_u_u.call.params.params_val[1] = expr;
		add_stmt(cast_new_stmt_expr(cex));
	}
}

void mach3_mu_state::mu_mapping_message_attribute(
	cast_expr expr,
	cast_type /*ctype*/,
	mint_ref itype,
	pres_c_mapping_message_attribute *attr_map) 
{
	assert((m(itype))->kind == MINT_VOID);
	switch (attr_map->kind) {
	case PRES_C_MESSAGE_ATTRIBUTE_FLAGS:
		if (op & MUST_ENCODE) {
			/* "_stream->msg_buf->Head.msgh_bits |= expr" */
			cast_expr attr_expr = cast_new_expr_op_assign(
				CAST_BINARY_BOR,
				cast_new_expr_name(
					"_stream->msg_buf->Head.msgh_bits"),
				expr);
			add_stmt(cast_new_stmt_expr(attr_expr));
		} else if (op & MUST_DECODE) {
			panic("message flags cannot be explicitly decoded!\n");
		}
		break;
	case PRES_C_MESSAGE_ATTRIBUTE_TIMEOUT:
		if (op & MUST_ENCODE) {
			/* "_stream->msg_buf->Head.msgh_bits |=
			   MACH_RCV_TIMEOUT" */
			cast_expr attr_expr = cast_new_expr_op_assign(
				CAST_BINARY_BOR,
				cast_new_expr_name(
					"_stream->msg_buf->Head.msgh_bits"),
				cast_new_expr_name(
					"MACH_RCV_TIMEOUT"));
			add_stmt(cast_new_stmt_expr(attr_expr));
		} else if (op & MUST_DECODE) {
			panic("A message timeout cannot be explicitly"
			      " decoded!\n");
		}
		break;
	
	case PRES_C_MESSAGE_ATTRIBUTE_SEQUENCE_RECEIVED:
		if (op & MUST_DECODE) {
			/* "expr = _stream->msg_buf->Head.msgh_seqno" */
			cast_expr attr_expr = cast_new_expr_assign(
				expr, cast_new_expr_name(
					"_stream->msg_buf->Head.msgh_seqno"));
			add_stmt(cast_new_stmt_expr(attr_expr));
		} else {
			panic("A message sequence number cannot be explicitly"
			      " encoded!\n");
		}
		break;
	
	default:
		panic("In `mach3_mu_state::mu_mapping_message_attribute', unknown attribute %d.",
		      attr_map->kind);
		break;
	}
}


void mach3_mu_state::mu_mapping_fixed_array(
	cast_expr expr,
	cast_type ctype,
	mint_ref itype,
	pres_c_mapping_fixed_array *array_mapping)
{
	/*
	 * XXX - this is an incomplete hack to distinguish between an
	 * array of characters versus a C-style string.
	 *
	 * There currently is no way for us to distinguish between a
	 * character belonging to a fixed array and a character
	 * belonging to a fixed-size C-style string.  MIG uses
	 * separate names for encoding these types of parameters,
	 * which ends up being the sole difference in the final
	 * message.  Flick doesn't currently support MIG's notion of a
	 * STRING_C type name.
	 */
	   
	/*
	 * If the CAST type of our array is a named type, locate the actual
	 * array/pointer type.  (Note: The MIG FE/PG makes these kinds of
	 * direct associations between named C types and MAPPING_FIXED_ARRAY
	 * nodes.  Other PG's generally indirect through a MAPPING_STUB, and
	 * the named type is derefenced to an array/pointer type there.)  */
	cast_type pctype = cast_find_typedef_type(&(pres->cast), ctype);
	if (!pctype)
		panic("In `mach3_mu_state::mu_mapping_fixed_array', "
		      "can't locate `typedef' for a named type.");
	
	/* strip off ptr for OOL data, if necessary */
	if (strcmp(array_mapping->alloc.allocator, "out_of_line") == 0) {
		if (pctype->kind == CAST_TYPE_ARRAY) {
			pctype = pctype->cast_type_u_u.array_type.element_type;
		} else {
			assert(pctype->kind == CAST_TYPE_POINTER);
			pctype = pctype->cast_type_u_u.pointer_type.target;
		}
		pctype = cast_find_typedef_type(&(pres->cast), ctype);
		if (!pctype)
			panic("In `mach3_mu_state::mu_mapping_fixed_array', "
			      "can't locate `typedef' for a named type.");
	}
	
	/* the MIG fe makes fixed-length strings much like fixed
           arrays, except that the ctype is a pointer instead of an
           array.  We check that here so we can output the correct MIG
           type in the message. */
	if ((pctype->kind == CAST_TYPE_POINTER)
	    && (pctype->cast_type_u_u.pointer_type.target->
		kind == CAST_TYPE_PRIMITIVE)
	    && (pctype->cast_type_u_u.pointer_type.target->
		cast_type_u_u.primitive_type.kind == CAST_PRIM_CHAR)) {
		is_fixed_string = 1;
	}
	mu_state::mu_mapping_fixed_array(expr, ctype, itype, array_mapping);
	is_fixed_string = 0;
}


void mach3_mu_state::get_prim_params(mint_ref itype, int *out_size, int *out_align_bits,
				     char **out_name)
{
	/* Booleans are handled as special 32-bit types under Mach 3.  */
	mint_def *def = &pres->mint.defs.defs_val[itype];

	if ((def->kind == MINT_INTEGER)
		   && (def->mint_def_u.integer_def.min == 0)
		   && (def->mint_def_u.integer_def.range == 1)) {
		/*
		 * If we didn't handle booleans specially,pbe_mem would marshal
		 * them in as little space as possible - i.e. one byte. 
		 * However, according to MIG conventions, booleans are 32 bits.
		 */
		*out_size = 4;
		*out_align_bits = 2;
		*out_name = flick_asprintf("flick_%s_%s_boolean",
					   get_be_name(), get_buf_name());
	} else
		/* Do the default thing */
		mem_mu_state::get_prim_params(itype, out_size, out_align_bits, out_name);

	if (!mach3_array) {
		/*
		 * When m/u-ing regular data, the data must take up at least
		 * 4 bytes, plus 4 bytes for the type header.
		 */
		if (*out_size < 4) *out_size = 4;
		*out_size += 4; /* for type header info */
	}
	*out_align_bits = 2; /* always */
}

void mach3_mu_state::mu_aggregated_array(
	cast_expr array_expr, cast_type array_ctype,
	pres_c_allocation *array_alloc,
	cast_type elem_ctype, mint_ref elem_itype,
	pres_c_mapping elem_map,
	cast_expr len_expr, cast_type len_ctype,
	unsigned long len_min, unsigned long len_max,
	mint_ref prim_itype, int fixed_elts,
	int out_of_line)
{
	mach3_array = 1;

	/* Decide whether to use a shortform or longform type descriptor.  */
	int longform = (len_max >= 4096);

	/* Find the vital parameters of the primitive array elements.  */
	int prim_size, prim_align_bits; char *prim_macro_name;
	get_prim_params(prim_itype, &prim_size, &prim_align_bits,
			&prim_macro_name);
#if 0
	/* XXX - this is an incomplete hack to distinguish between an
	   array of characters versus a C-style string.
	   See comment in mu_mapping_fixed_array for more info. */
	   
	/* Output the proper tag for fixed-length strings */
	if (is_fixed_string) {
		prim_macro_name = flick_asprintf("flick_%s_%s_string_c",
						 get_be_name(),
						 get_buf_name());
		is_fixed_string = 0;
	}
#endif
	/* Produce a macro invocation to marshal the type descriptor.  */
	char *macro_name = flick_asprintf("%s_type", prim_macro_name);
	cast_expr macro_expr = cast_new_expr_name(macro_name);
	cast_expr ofs_expr = cast_new_expr_lit_int(
		chunk_prim(2, longform ? 12 : 4), 0);
	cast_expr num_expr;
	if (fixed_elts > 1) {
		num_expr = cast_new_binary_expr(
			CAST_BINARY_MUL,
			len_expr,
			cast_new_expr_lit_int(fixed_elts, 0));
		if (op & MUST_DECODE) {
			add_stmt(
				cast_new_stmt_expr(
					cast_new_expr_assign(
						len_expr,
						num_expr)));
			num_expr = len_expr;
		}
	} else 
		num_expr = len_expr;
	cast_expr inl_expr = cast_new_expr_lit_int(!out_of_line, 0);
	cast_expr long_expr = cast_new_expr_name(
		longform ? "_long_type" : "_type");
	
	cast_expr cex;
	if (pres->mint.defs.defs_val[prim_itype].kind == MINT_INTERFACE) {
		assert(elem_map->kind == PRES_C_MAPPING_REFERENCE);
		cex = cast_new_expr_call(macro_expr, 5);
		cex->cast_expr_u_u.call.params.params_val[0] = ofs_expr;
		cex->cast_expr_u_u.call.params.params_val[1] =
			cast_new_expr_name(
				get_mach_port_type(
					&elem_map->pres_c_mapping_u_u.ref,
					prim_itype));;
		cex->cast_expr_u_u.call.params.params_val[2] = num_expr;
		cex->cast_expr_u_u.call.params.params_val[3] = inl_expr;
		cex->cast_expr_u_u.call.params.params_val[4] = long_expr;
		is_complex = 1;
	} else {
		cex = cast_new_expr_call(macro_expr, 4);
		cex->cast_expr_u_u.call.params.params_val[0] = ofs_expr;
		cex->cast_expr_u_u.call.params.params_val[1] = num_expr;
		cex->cast_expr_u_u.call.params.params_val[2] = inl_expr;
		cex->cast_expr_u_u.call.params.params_val[3] = long_expr;
	}
	add_stmt(cast_new_stmt_expr(cex));
	
	/* do bounds checking, if necessary */
	if (len_max != 0xffffffff && len_max != len_min) {
		add_stmt(cast_new_if(
			cast_new_binary_expr(CAST_BINARY_GT,
					     len_expr,
					     cast_new_expr_lit_int(
						     len_max,
						     CAST_MOD_UNSIGNED)
				),
			cast_new_stmt_expr(cast_new_expr_name(
				"return MIG_ARRAY_TOO_LARGE")),
			0));
	}
	
	/* Now descend into the array and marshal the individual elements. */
	
	mem_mu_state::mu_array(array_expr, array_ctype, array_alloc,
			       elem_ctype, elem_itype, elem_map,
			       len_expr, len_ctype, len_min, len_max);
	
	mach3_array = 0;
	
	/* we need to align ourselves to a 4-byte boundary */
	if ((align_bits < 2) || (align_ofs & 3))
		chunk_prim(2, 0);
}

void mach3_mu_state::mu_array(
	cast_expr array_expr, cast_type array_ctype,
	pres_c_allocation *array_alloc,
	cast_type elem_ctype, mint_ref elem_itype,
	pres_c_mapping elem_map,
	cast_expr len_expr, cast_type len_ctype,
	unsigned long len_min, unsigned long len_max)
{
	/* Decide whether to transmit it in-line or out-of-line. */
	int out_of_line = 0;
	if (strcmp(array_alloc->allocator, "out_of_line") == 0) {
		out_of_line = 1;
		is_complex = 1;
	}
	
	/* See if the whole array can be lumped under one MIG type descriptor.
	   This is possible iff this is an array of [fixed arrays of...]
	   primitive types.  At the same time, compute the total number of
	   primitive elements contained in each element of this (top-level,
	   possibly variable-length) array.  */
	mint_ref prim_itype = elem_itype;
	int fixed_elts = 1;

	if (out_of_line || len_ctype) while (1) {
		assert(prim_itype >= 0);
		assert(prim_itype < (signed int)pres->mint.defs.defs_len);
		
		mint_def *def = &pres->mint.defs.defs_val[prim_itype];

		/* Don't try to aggregate again if we're already aggregating. */
		if (mach3_array)
			break;
		
		if ((def->kind == MINT_INTEGER) ||
		    (def->kind == MINT_FLOAT) ||
		    (def->kind == MINT_CHAR) ||
		    (def->kind == MINT_INTERFACE)) {
			/* We reached the primitive type at the bottom,
			   so aggregation is possible - do it.  */
			mu_aggregated_array(array_expr, array_ctype,
					    array_alloc,
					    elem_ctype, elem_itype, elem_map,
					    len_expr, len_ctype,
					    len_min, len_max,
					    prim_itype, fixed_elts,
					    out_of_line);
			return;
			
		} else if (def->kind == MINT_ARRAY) {
			unsigned array_min, array_max;
			mint_get_array_len(&pres->mint, prim_itype,
					   &array_min, &array_max);
			if (array_min != array_max) {
				/* Can't aggregate arrays of variable-length
                                   arrays.*/
				break;
			}
			fixed_elts *= array_min;
			/* simple overflow check */
			assert(fixed_elts % array_min == 0);
			prim_itype = def->mint_def_u.array_def.element_type;
		} else {
			/* Saw something else - no aggregation possible.  */
#if 0
			/*
			 * XXX - presentations other than MIG may cause this
			 * case to occur, so we can't panic here... But for
			 * MIG arrays, we have to know what type to send for
			 * the type tag.  If we get here, we don't know, and
			 * thus can't make a correct tag.  It would be wrong
			 * to handle it "the normal way", because no tag would
			 * be associated with this array, and the message
			 * would be (at least conceptually) wrong.
			 */
			if (len_max == 1)
				panic("Can't determine type of pointer data");
#endif
			break;
		}
	}

	/* No aggregation possible - deal with the array the normal way.  */
	mem_mu_state::mu_array(array_expr, array_ctype, array_alloc,
			       elem_ctype, elem_itype, elem_map,
			       len_expr, len_ctype, len_min, len_max);	
}

mach3_target_mu_state::mach3_target_mu_state(pres_c_1 *pres, mu_state_op op,
					     int assumptions, char *which)
	: target_mu_state(pres, op, assumptions, which)
{
}

mach3_target_mu_state::mach3_target_mu_state(const mach3_target_mu_state &must)
	: target_mu_state(must)
{
}

mu_state *mach3_target_mu_state::clone()
{
	return new mach3_target_mu_state(*this);
}

char *mach3_target_mu_state::get_be_name()
{
	return "mach3mig";
}

char *mach3_target_mu_state::get_encode_name()
{
	return "mach3mig";
}

char *mach3_mu_state::get_be_name()
{
	if (mach3_array)
		return "mach3mig_array";
	else
		return "mach3mig";
}

char *mach3_mu_state::get_encode_name()
{
	if (mach3_array)
		return "mach3mig_array";
	else
		return "mach3mig";
}

void w_header_includes(pres_c_1 * /*p*/)
{
	w_printf("#include <flick/link/mach3mig.h>\n\n");
	w_printf("#include <flick/encode/mach3mig.h>\n\n");
}

/*
 * `remove_idl_and_interface_ids' strips away the ``collapsed union'' stuff
 * that encodes IDL and interface information.  Mach3MIG client and server
 * stubs don't need to encode this information because it is manifest in the
 * object references (ports).
 *
 * Similarly, `remove_operation_id' strips away the ``collapsed union'' stuff
 * that encodes an operation identifier.  This information isn't required in
 * reply messages.
 */

void remove_idl_and_interface_ids(
	pres_c_1 *pres,
	mint_ref in_itype, pres_c_inline in_inline,
	mint_ref *out_itype, pres_c_inline *out_inline)
{
	*out_itype  = in_itype;
	*out_inline = in_inline;
	
	descend_collapsed_union(pres, out_itype, out_inline);
	descend_collapsed_union(pres, out_itype, out_inline);
}

void remove_operation_id(
	pres_c_1 *pres,
	mint_ref in_itype, pres_c_inline in_inline,
	mint_ref *out_itype, pres_c_inline *out_inline)
{
	*out_itype  = in_itype;
	*out_inline = in_inline;
	
	descend_collapsed_union(pres, out_itype, out_inline);
}

void mach3_mu_state::set_id_expected(
	mint_ref operation_union_iref)
{
	mint_def *operation_union_idef = m(operation_union_iref);
	mint_ref discrim_iref;
	
	if (operation_union_idef->kind == MINT_UNION) {
		/* Dig the discriminator out. */
		discrim_iref = operation_union_idef->
			       mint_def_u.union_def.discrim;
		
		if ((m(discrim_iref))->kind == MINT_INTEGER)
			/*
			 * The operation ID is an integer, so we encode it in
			 * the `msgh_id' field of the message headers.
			 */
			id_expected = 2;
		else
			/*
			 * The operation identifier is too complicated to
			 * encode in the Mach message header.
			 */
			id_expected = 0;
	} else
		/*
		 * We don't even have a union?!
		 */
		id_expected = 0;
}

void mach3_mu_state::mu_end()
{
	/* we need to align ourselves to a 4-byte boundary */
	if ((align_bits < 2) || (align_ofs & 3))
		chunk_prim(2, 0);

	/* now do the default thing */
	mem_mu_state::mu_end();
}

/* End of file. */

