/*
 * Copyright (c) 1997 The University of Utah and
 * the Computer Systems Laboratory at the University of Utah (CSL).
 *
 * This file is part of Flick, the Flexible IDL Compiler Kit.
 *
 * Flick is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Flick is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Flick; see the file COPYING.  If not, write to
 * the Free Software Foundation, 59 Temple Place #330, Boston, MA 02111, USA.
 */

#include <mom/c/libcast.h>

#include "pg_fluke.hh"

/*
 * `p_server_skel_cdef' allocates and returns an index to the CAST declaration
 * of a server skeleton (a.k.a. a server dispatch function).
 */
cast_ref pg_fluke::p_server_skel_cdef(char *sskel_name)
{
	cast_ref        sskel_cref = cast_add_func_def(&(out_pres->cast),
						       sskel_name,
						       2);
	cast_func_type *sskel_type = &(out_pres->
				       cast.cast_scope_val[sskel_cref].
				       u.cast_def_u_u.func_type);
	
	/*
	 * A MOM server skeleton (a.k.a. dispatch function) has the type:
	 *
	 *   mom_dispatch_action_t <name>(void *_dispatch_object,
	 *                                mom_dispatch_data_t *_dispatch_data);
	 */
	
	/* Set the return type. */
	sskel_type->return_type = cast_new_type_name("mom_dispatch_action_t");
	
	/* Set the parameter names and types. */
	sskel_type->params.params_val[0].name = "_dispatch_object";
	sskel_type->params.params_val[0].type =
		cast_new_pointer_type(
			cast_new_type(CAST_TYPE_VOID));
	
	sskel_type->params.params_val[1].name = "_dispatch_data";
	sskel_type->params.params_val[1].type =
		cast_new_pointer_type(
			cast_new_type_name("mom_dispatch_data_t"));

	return sskel_cref;
}

/* End of file. */

