/*
 * Copyright (c) 1996, 1997 The University of Utah and
 * the Computer Systems Laboratory at the University of Utah (CSL).
 * All rights reserved.
 *
 * Permission to use, copy, modify and distribute this software is hereby
 * granted provided that (1) source code retains these copyright, permission,
 * and disclaimer notices, and (2) redistributions including binaries
 * reproduce the notices in supporting documentation, and (3) all advertising
 * materials mentioning features or use of this software display the following
 * acknowledgement: ``This product includes software developed by the
 * Computer Systems Laboratory at the University of Utah.''
 *
 * THE UNIVERSITY OF UTAH AND CSL ALLOW FREE USE OF THIS SOFTWARE IN ITS "AS
 * IS" CONDITION.  THE UNIVERSITY OF UTAH AND CSL DISCLAIM ANY LIABILITY OF
 * ANY KIND FOR ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 *
 * CSL requests users of this software to return to csl-dist@cs.utah.edu any
 * improvements that they make and grant CSL redistribution rights.
 */

#ifndef _flick_link_ORB_h
#define _flick_link_ORB_h

/* This is a bit of stuff to try to implement a more ORBey orb */

typedef struct CORBA_ORB_type *CORBA_ORB;
typedef struct CORBA_BOA_type *CORBA_BOA;

struct CORBA_ORB_type {
        struct sockaddr_in *ipaddr;
	char *hostname;
	char *ORBid;
	int OA_count;
	CORBA_BOA *boas;
};

struct CORBA_BOA_type {
        FLICK_BUFFER *in, *out;
        CORBA_ORB orb;
        unsigned short hostport;
	int socket_fd;
	char *OAid;

	/* This is a count of refs on a server,
	   on a client, it's the CORBA_Object ref count */
        int count_servers;
	/* This is for servers */
	FLICK_CLIENT refs;

	/* This is only used from within the client object list above */
        FLICK_SERVER server_func;
	int max_name_len;
};

/*
 * `IOP' stuff that we need in order to deal with CORBA Interoperable Object
 * References (IORs).  See Section 10.6.2 of the CORBA 2.0 specification.
 */

typedef /* CORBA_unsigned_long */ unsigned long IOP_ProfileId;
#define IOP_TAG_INTERNET_IOP (0)
#define IOP_TAG_MULTIPLE_COMPONENTS (1)

/*
 * Stuff for ORB/BOA conservation...
 */
extern CORBA_ORB *orbs;
extern int orb_count;

/********
 * Here's the ORB interface that applies (non-DII/DSI/IR stuff...)
 ********/
CORBA_ORB CORBA_ORB_init(int argc, char **argv);			/* Checked */
CORBA_BOA CORBA_ORB_BOA_init(CORBA_ORB ths, int argc, char **argv);	/* Checked */
/* This thing currently only tries the first BOA environment created
 * see CORBA_BOA_bind for a better version...
 */
CORBA_Object CORBA_ORB_resolve_initial_references(CORBA_ORB ths, char *key); /* Checked */
char *CORBA_ORB_object_to_readable_string(CORBA_Object obj);		/* Checked */
char *CORBA_ORB_object_to_string(CORBA_Object obj);			/* Checked */
CORBA_Object CORBA_ORB_string_to_object(char *str);			/* Checked */

/********
 * Here's the BOA stuff that applies...
 ********/
/* This is my CORBA_BOA_create equivalent that doesn't require an IR */
CORBA_Object CORBA_BOA_bind(CORBA_BOA ths, const char *type_id, CORBA_ReferenceData *key);	/* Checked */
CORBA_ReferenceData *CORBA_BOA_get_id(CORBA_BOA ths, CORBA_Object obj);	/* Checked */

void CORBA_BOA_obj_is_ready(CORBA_BOA ths, const char *type_id, FLICK_SERVER, CORBA_ReferenceData *key);	/* Checked */
void CORBA_BOA_impl_is_ready(CORBA_BOA ths);				/* Checked */

/* This is here because we're not doing char id's yet... */
#define CORBA_BOA_set_exception(boa, ev_ptr, mjr, id, data) {	\
	(ev_ptr)->_major = (mjr);				\
	(ev_ptr)->_atomic_user_id = (id);			\
	(ev_ptr)->_except = (void *)(data);			\
}

/*
 * I'm not doing this stuff right now:
 *
 * void CORBA_BOA_dispose(CORBA_BOA ths, CORBA_Object obj);
 */

/*
 * The various CORBA::Object methods are now in `flick/pres/corba.h', because
 * they must be defined after the basic CORBA types are defined (for example,
 * `CORBA_boolean').
 *
 * XXX --- Arguably, *all* of the ORB methods are part of the presentation, and
 * should be moved into the presentation headers.
 */

#endif /* _flick_link_ORB_h */

/* End of file. */

