/*
 * Copyright (c) 1996, 1997 The University of Utah and
 * the Computer Systems Laboratory at the University of Utah (CSL).
 * All rights reserved.
 *
 * Permission to use, copy, modify and distribute this software is hereby
 * granted provided that (1) source code retains these copyright, permission,
 * and disclaimer notices, and (2) redistributions including binaries
 * reproduce the notices in supporting documentation, and (3) all advertising
 * materials mentioning features or use of this software display the following
 * acknowledgement: ``This product includes software developed by the
 * Computer Systems Laboratory at the University of Utah.''
 *
 * THE UNIVERSITY OF UTAH AND CSL ALLOW FREE USE OF THIS SOFTWARE IN ITS "AS
 * IS" CONDITION.  THE UNIVERSITY OF UTAH AND CSL DISCLAIM ANY LIABILITY OF
 * ANY KIND FOR ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 *
 * CSL requests users of this software to return to csl-dist@cs.utah.edu any
 * improvements that they make and grant CSL redistribution rights.
 */

#ifndef __flick_link_mach3mig_h
#define __flick_link_mach3mig_h

#include <flick/link/all.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <mach.h>
#include <mach/mig_errors.h>
#include <mach/mig_support.h>
#include <mach/port.h>
#include <mach/mach_port.h>

/*#include <flick/pres/mig.h>*/

#define FLICK_NO_MEMORY		499 /*XXX*/

extern void *__buffer;
extern int   __buflen;

/******************************************************************************
 ******************************************************************************
 ****
 **** Internal Flick data types
 ****
 ******************************************************************************
 *****************************************************************************/

#define msgh_request_port	msgh_local_port
#define MACH_MSGH_BITS_REQUEST(bits)	MACH_MSGH_BITS_LOCAL(bits)
#define msgh_reply_port		msgh_remote_port
#define MACH_MSGH_BITS_REPLY(bits)	MACH_MSGH_BITS_REMOTE(bits)

struct flick_port_list {
	mach_port_t p;
	struct flick_port_list *next;
};

struct flick_marshal_stream_type {
	/* This is initially set to point to init_buf,
	   but is dynamically re-allocated if more space is needed. */
	mig_reply_header_t *msg_buf;
	void *msg_end;

	/* The pointer to the current chunk. */
	void *chunk_ptr;

	/* This is for auto-freeing of ports received as in parameters */
	struct flick_port_list *ports_to_free;
};
typedef struct flick_marshal_stream_type *flick_marshal_stream, *flick_marshal_stream_t, FLICK_BUFFER;

typedef mach_port_t FLICK_CLIENT;

typedef boolean_t (FLICK_SERVER)(mach_msg_header_t *InHeadP, mach_msg_header_t *OutHeadP);
typedef FLICK_SERVER flick_dispatch_t;

/******************************************************************************
 ******************************************************************************
 ****
 **** Encoding
 ****
 ******************************************************************************
 *****************************************************************************/

#define flick_mach3mig_encode_target(_data, _adjust)			\
{									\
	if ((_adjust) > 1) {						\
		if (mach_port_mod_refs(mach_task_self(), (_data),	\
			MACH_PORT_RIGHT_SEND, -((_adjust)-1)))		\
		  return _return;					\
	}								\
	_stream->msg_buf->Head.msgh_remote_port = (_data);		\
	_stream->msg_buf->Head.msgh_bits =				\
		MACH_MSGH_BITS((_adjust) ?				\
					MACH_MSG_TYPE_MOVE_SEND		\
						:			\
					MACH_MSG_TYPE_COPY_SEND,	\
			       0);					\
	_stream->chunk_ptr += sizeof(_stream->msg_buf->Head);		\
}

#define flick_mach3mig_encode_new_glob_plain(max_size)		\
	flick_mach3mig_encode_new_glob(max_size)
#define flick_mach3mig_encode_end_glob_plain(max_size)		\
	flick_mach3mig_encode_end_glob(max_size)
#define flick_mach3mig_encode_new_chunk_plain(size)		\
	flick_mach3mig_encode_new_chunk(size)
#define flick_mach3mig_encode_end_chunk_plain(size)		\
	flick_mach3mig_encode_end_chunk(size)

/* Primitive types with individual type descriptors.  */

int flick_mach3mig_rpc_grow_buf(flick_marshal_stream, int);

#define flick_mach3mig_encode_new_glob(max_size)		\
{								\
  if (_stream->chunk_ptr + (max_size) > _stream->msg_end)	\
  {								\
    if (flick_mach3mig_rpc_grow_buf(_stream, max_size)		\
	!= MACH_MSG_SUCCESS)					\
      exit(1);							\
  }								\
}

#define flick_mach3mig_encode_end_glob(max_size) /* Do nothing. */

/*****/

#define flick_mach3mig_encode_new_chunk(size)	/* Do nothing. */
#define flick_mach3mig_encode_new_chunk_align(size, ofs, a, b)		\
{									\
	int __align = 1 << (ofs);					\
	_stream->chunk_ptr +=						\
		((__align - (((int)_stream->chunk_ptr) &		\
			     (__align - 1))) &				\
		 (__align - 1));					\
}

#define flick_mach3mig_encode_end_chunk(size) (_stream->chunk_ptr += (size))

/*****/

/* Array elements.  */
#define flick_mach3mig_array_encode_new_glob(max_size) \
	flick_mach3mig_encode_new_glob(max_size)
#define flick_mach3mig_array_encode_end_glob(max_size) \
	flick_mach3mig_encode_end_glob(max_size)
#define flick_mach3mig_array_encode_new_chunk(size) \
	flick_mach3mig_encode_new_chunk(size)
#define flick_mach3mig_array_encode_end_chunk(size) \
	flick_mach3mig_encode_end_chunk(size)

/******************************************************************************
 ******************************************************************************
 ****
 **** Decoding
 ****
 ******************************************************************************
 *****************************************************************************/

#if TypeCheck
#define flick_iftypecheck(code) code
#else
#define flick_iftypecheck(code)
#endif

/* Primitive types with individual type descriptors.  */
#define flick_mach3mig_decode_new_glob(max_size) /* Do nothing. */
#define flick_mach3mig_decode_end_glob(max_size) /* Do nothing. */

#define flick_mach3mig_decode_new_chunk(size)				\
{									\
	flick_iftypecheck(						\
		if (_stream->chunk_ptr + (size) > _stream->msg_end)	\
			return MIG_TYPE_ERROR;				\
	);								\
}
#define flick_mach3mig_decode_new_chunk_align(size, ofs, a, b) {	\
    int __align = 1 << (ofs);						\
    _stream->chunk_ptr +=						\
	((__align - (((int)_stream->chunk_ptr) & (__align - 1))) &	\
	 (__align - 1));						\
  }
#define flick_mach3mig_decode_end_chunk(size) (_stream->chunk_ptr += (size))

/* Array elements.  */
#define flick_mach3mig_array_decode_new_glob(max_size)	flick_mach3mig_decode_new_glob(max_size)
#define flick_mach3mig_array_decode_end_glob(max_size)	flick_mach3mig_decode_end_glob(max_size)
#define flick_mach3mig_array_decode_new_chunk(size)	flick_mach3mig_decode_new_chunk(size)
#define flick_mach3mig_array_decode_end_chunk(size)	flick_mach3mig_decode_end_chunk(size)


/******************************************************************************
 ******************************************************************************
 ****
 **** Client-side support
 ****
 ******************************************************************************
 *****************************************************************************/

mach_msg_return_t flick_mach3mig_rpc(flick_marshal_stream rpc);

mach_msg_return_t flick_mach3mig_send(flick_marshal_stream send);


/******************************************************************************
 ******************************************************************************
 ****
 **** Server-side support
 ****
 ******************************************************************************
 *****************************************************************************/

#define flick_mach3mig_server_start_decode()						\
	_stream = &_temp;								\
	_stream->msg_buf = (mig_reply_header_t*)InHeadP;				\
	_stream->msg_end = ((char *)InHeadP) + InHeadP->msgh_size;			\
	_stream->chunk_ptr = (void *)_stream->msg_buf;					\
	_stream->ports_to_free = 0;

#define flick_mach3mig_server_end_decode() /* Do nothing. */

#define flick_mach3mig_server_start_encode()						\
	_stream->msg_buf = (mig_reply_header_t*)OutHeadP;				\
	_stream->msg_end = ((char *)OutHeadP) + 65536;/*OutHeadP->msgh_size;*/		\
	_stream->chunk_ptr = (void *)_stream->msg_buf;					\
	/* XXX --- _stream->ports_to_free = 0; --- wrong to do this here! */

#define flick_mach3mig_server_end_encode()									\
{														\
	mach_msg_type_t _ret_tmpl = { MACH_MSG_TYPE_INTEGER_32, 32, 1, 1, 0, 0 };				\
	OutHeadP->msgh_bits = MACH_MSGH_BITS(MACH_MSGH_BITS_REPLY(InHeadP->msgh_bits), 0);			\
	OutHeadP->msgh_size = (_stream->chunk_ptr - ((void *)_stream->msg_buf));				\
	OutHeadP->msgh_remote_port = InHeadP->msgh_remote_port;							\
	OutHeadP->msgh_local_port = MACH_PORT_NULL;								\
        _stream->msg_buf->RetCodeType = _ret_tmpl;								\
	while (_stream->ports_to_free) {									\
	  mach_port_deallocate(mach_task_self(), _stream->ports_to_free->p);					\
	  _stream->ports_to_free = _stream->ports_to_free->next;						\
	}													\
}

#define flick_mach3mig_client_start_encode()				\
	_stream = &_temp;						\
	_stream->msg_buf = (mig_reply_header_t *)__buffer;		\
	_stream->chunk_ptr = (void *)_stream->msg_buf;			\
	_stream->msg_end = (void *)((char *)_stream->msg_buf + __buflen);

#define flick_mach3mig_client_end_encode()

#define flick_mach3mig_client_start_decode()						\
	_stream->chunk_ptr = (void *)_stream->msg_buf + sizeof(mig_reply_header_t);

#define flick_mach3mig_client_end_decode() {}

#define flick_mach3mig_client_encode_target(_target, _size, _link) {	 \
	flick_##_link##_encode_new_glob(sizeof(mach_msg_header_t)); \
	flick_##_link##_encode_new_chunk(sizeof(mach_msg_header_t));\
	_stream->msg_buf->Head.msgh_remote_port = (_target);		 \
	_stream->msg_buf->Head.msgh_seqno = 0;		 		 \
	flick_##_link##_encode_end_chunk(sizeof(mach_msg_header_t));\
	flick_##_link##_encode_end_glob(sizeof(mach_msg_header_t)); \
}

#define flick_mach3mig_server_decode_target(_target, _size, _link) {	 \
	flick_##_link##_encode_new_glob(sizeof(mach_msg_header_t)); \
	flick_##_link##_encode_new_chunk(sizeof(mach_msg_header_t));\
	(_target) = _stream->msg_buf->Head.msgh_remote_port;		 \
	flick_##_link##_encode_end_chunk(sizeof(mach_msg_header_t));\
	flick_##_link##_encode_end_glob(sizeof(mach_msg_header_t)); \
}

#define flick_mach3mig_server_encode_target(_target, _size, _link)	\
	flick_##_link##_encode_new_glob(sizeof(mig_reply_header_t)); \
	flick_##_link##_encode_new_chunk(sizeof(mig_reply_header_t));\
	_stream->msg_buf->Head.msgh_remote_port = (_target);		 \
	_stream->msg_buf->Head.msgh_seqno = 0;		 		 \
	flick_##_link##_encode_end_chunk(sizeof(mig_reply_header_t));\
	flick_##_link##_encode_end_glob(sizeof(mig_reply_header_t)); \

#endif /* __flick_link_mach3mig_h */

/* End of file. */

