/*
 * Copyright (c) 1996, 1997 The University of Utah and
 * the Computer Systems Laboratory at the University of Utah (CSL).
 * All rights reserved.
 *
 * Permission to use, copy, modify and distribute this software is hereby
 * granted provided that (1) source code retains these copyright, permission,
 * and disclaimer notices, and (2) redistributions including binaries
 * reproduce the notices in supporting documentation, and (3) all advertising
 * materials mentioning features or use of this software display the following
 * acknowledgement: ``This product includes software developed by the
 * Computer Systems Laboratory at the University of Utah.''
 *
 * THE UNIVERSITY OF UTAH AND CSL ALLOW FREE USE OF THIS SOFTWARE IN ITS "AS
 * IS" CONDITION.  THE UNIVERSITY OF UTAH AND CSL DISCLAIM ANY LIABILITY OF
 * ANY KIND FOR ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 *
 * CSL requests users of this software to return to csl-dist@cs.utah.edu any
 * improvements that they make and grant CSL redistribution rights.
 */

#include "iiop-link.h"

/* needed only for flick_check_byte_order */
#include <flick/encode/cdr.h>

const int one = 1;
char flick_is_little_endian = -1; /* not defined yet */

int flick_seqnum = 0;

CORBA_ORB *orbs = 0;
int orb_count = 0;

CORBA_Object CORBA_ORB_resolve_initial_references(CORBA_ORB ths, char *name)
{
	CORBA_ReferenceData key;
	key._length = strlen(name);
	key._buffer = name;
	key._maximum = 1024;
	if (!ths->boas)
		return 0;
	return CORBA_BOA_bind(ths->boas[0], name, &key);
}

CORBA_ORB CORBA_ORB_init(int argc, char **argv)
{
	int i = 1;
	char *ORBaddr = 0;
	char *ORBid = 0;
	CORBA_ORB orb = 0;
	char buf[256];
	struct hostent *he;
	
	/*****/
	
	flick_is_little_endian = *(char *)(&one);
	flick_check_byte_order();
	
	for (i = 1; i < argc; i++) {
		if (!strcmp(argv[i], "-ORBipaddr") &&
		    i + 1 < argc) {
			if (!ORBaddr)
				ORBaddr = argv[++i];
			else {
				fprintf(stderr,
					"Error: too many ORB addresses "
					"specified.\n");
				return 0;
			}
		} else if (!strcmp(argv[i], "-ORBid") &&
			   i + 1 < argc) {
			if (!ORBid)
				ORBid = argv[++i];
			else {
				fprintf(stderr,
					"Error: too many ORB identifiers "
					"specified.\n");
				return 0;
			}
		}
	}
	if (!ORBaddr) {
		if (gethostname(buf, 256)) {
			fprintf(stderr,
				"Error: unable to get name of local host.\n");
			return 0;
		}
		ORBaddr = &buf[0];
		fprintf(stderr,
			"Warning: no `-ORBipaddr' specified; using `%s'.\n",
			ORBaddr);
	}
	if (!ORBid)
		ORBid = "FlickORB--";
	
	if (!(he = gethostbyname(ORBaddr))) {
		fprintf(stderr,
			"Error: unable to identify host `%s'.\n",
			ORBaddr);
		return 0;
	}
	
	if (orb_count) {
		int i;
		/*
		 * Let's see if we already have an ORB connection to the
		 * specified machine.
		 */
		for (i = 0; i < orb_count; i++)
			if (!strcmp(he->h_name, orbs[i]->hostname))
				return orbs[i];
	}
	orb = t_calloc(struct CORBA_ORB_type, 1);
	if (!orb) {
		fprintf(stderr, "Error: unable to allocate space for ORB.\n");
		goto err1;
	}
	
	orb->ipaddr = t_calloc(struct sockaddr_in, 1);
	if (!orb->ipaddr) {
		fprintf(stderr,
			"Error: unable to allocate space for ORB IP "
			"address.\n");
		goto err2;
	}
	orb->ipaddr->sin_addr.s_addr = *(long*)*he->h_addr_list;
	orb->ipaddr->sin_family = AF_INET;
	orb->ipaddr->sin_port = 0;
	orb->hostname = t_malloc(char, strlen(he->h_name) + 1);
	if (!orb->hostname) {
		fprintf(stderr,
			"Error: unable to allocate space for hostname in "
			"ORB.\n");
		goto err3;
	}
	/*
	 * Use the FQDN returned by `gethostbyname', not `ORBaddr', because
	 * `ORBaddr' may not be fully qualified.
	 */
	strcpy(orb->hostname, he->h_name);
	
	orb->ORBid = t_malloc(char, strlen(ORBid) + 1);
	if (!orb->ORBid) {
		fprintf(stderr,
			"Error: unable to allocate space for ORB ID.\n");
		goto err4;
	}
	strcpy(orb->ORBid, ORBid);
	orb->OA_count = 0;
	orb->boas = 0;
	orbs = t_realloc(orbs, CORBA_ORB, ++orb_count);
	orbs[orb_count - 1] = orb;
	return orb;
err4:
	free(orb->hostname);
err3:
	free(orb->ipaddr);
err2:
	free(orb);
err1:
	return 0;
}

CORBA_BOA CORBA_ORB_BOA_init(CORBA_ORB ths, int argc, char **argv)
{
	int i;
	char *prt = 0;
	CORBA_BOA boa = 0;
	char *OAid = 0;
	unsigned short host_port;
	for (i = 1; i < argc; i++) {
		if (!strcmp(argv[i], "-OAport") &&
		    i + 1 < argc) {
			if (!prt)
				prt = argv[++i];
			else {
				fprintf(stderr,
					"Error: too many OA ports "
					"specified.\n");
				return 0;
			}
		} else if (!strcmp(argv[i], "-OAid") &&
			   i + 1 < argc) {
			if (!OAid)
				OAid = argv[++i];
			else {
				fprintf(stderr,
					"Error: too many OA id's "
					"specified.\n");
				return 0;
			}
		}
	}
	if (!OAid)
		OAid = "FlickBOA--";
	if (!prt) {
		fprintf(stderr, "Error: No `-OAport' specified.\n");
		return 0;
	}
	host_port = atoi(prt);
	if (ths->OA_count) {
		int i;
		/* Check to see if this BOA is already here... */
		for (i = 0; i < ths->OA_count; i++)
			if (ths->boas[i]->hostport == host_port &&
			    !strcmp(ths->boas[i]->OAid, OAid))
				return ths->boas[i];
	}
	
	boa = t_calloc(struct CORBA_BOA_type, 1);
	if (!boa)
		goto err0;
	
	boa->hostport = host_port;
	boa->orb = ths;
	boa->in = t_calloc(FLICK_BUFFER, 1);
	if (!boa->in)
		goto err1;

	boa->out = t_calloc(FLICK_BUFFER, 1);
	if (!boa->out)
		goto err2;
	boa->hostport = host_port;
	boa->orb->ipaddr->sin_port = htons(host_port);
	boa->count_servers = 0;
	boa->refs = 0;
	boa->socket_fd = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP);
	if (boa->socket_fd < 0)
		goto err3;
	
	boa->OAid = t_malloc(char, strlen(OAid) + 1);
	if (!boa->OAid)
		goto err4;
	strcpy(boa->OAid, OAid);
	
	ths->boas = t_realloc(ths->boas, CORBA_BOA, ++ths->OA_count);
	ths->boas[ths->OA_count - 1] = boa;
	
	boa->in->real_buf_start = boa->in->buf_read = boa->in->buf_start
				= boa->in->buf_current = t_calloc(char, 8192);
	if (!boa->in->buf_start)
		goto err4;
	
	boa->in->real_buf_end = boa->in->buf_end = boa->in->buf_start + 8192;
	
	boa->out->real_buf_start = boa->out->real_buf_end
				 = boa->out->buf_read = 0;
	boa->out->buf_start = boa->out->buf_current = t_calloc(char, 8192);
	if (!boa->out->buf_start)
		goto err5;
	
	boa->out->buf_end = boa->out->buf_start + 8192;
	
	return boa;
  err5:
	free(boa->in->real_buf_start);
  err4:
	close(boa->socket_fd);
  err3:
	free(boa->out);
  err2:
	free(boa->in);
  err1:
	free(boa);
  err0:
	return 0;
}

char *CORBA_ORB_object_to_readable_string(CORBA_Object obj)
{
	char buf[1024], *res;
	int text = 1, pos;
	if (!obj)
		return t_calloc(char, 1);
	for (pos = 0; pos < obj->key._length && text; pos++)
		text = obj->key._buffer[pos] >= ' ' && obj->key._buffer[pos] <= 126;
	if (!text)
		return CORBA_ORB_object_to_string(obj);
	sprintf(buf, "iiop:1.0//%s:%d/%s/",
		obj->boa->orb->hostname,
		obj->boa->hostport,
		obj->type_id);
	pos = strlen(buf);
	memcpy(&buf[pos], obj->key._buffer, obj->key._length);
	buf[pos + obj->key._length] = 0;
	res = t_malloc(char, strlen(buf) + 1);
	strcpy(res, buf);
	return res;
}

/*
 * XXX --- The IIOP runtime should cache objects' IORs so that we can just
 * blast them out.
 *
 * XXX --- If there is a bug in this code, you should also check `flick_cdr_
 * encode_IOR_internal' to see if has the same bug.  Someday we should combine
 * the marshaling guts of these two functions.
 */
char *CORBA_ORB_object_to_string(CORBA_Object obj)
{
	char buffer[1024], *res;
	unsigned int buffer_len;
	unsigned int pos, len;
	unsigned int i;
	
	unsigned int endian_test = 1;
	char native_endianness = ((char *) &endian_test)[0];
	
	/* Clear the buffer so we don't have to zero out pad bytes. */
	memset(buffer, 0, sizeof(buffer));
	pos = 0;
	
	/* Set the `endianness' flag on this encapsulation. */
	buffer[pos] = native_endianness;
	pos += 4;
	
	/* Encode the object `type_id'. */
	if (obj) {
		len = obj->type_id_len + 1;
		*(unsigned int *) &buffer[pos] = len;
		pos += 4;
		strncpy((&buffer[pos]), obj->type_id, len);
		pos += (len + 3) & ~3;
	} else {
		/* Null object references have an empty `type_id' string. */
		*(unsigned int *) &buffer[pos] = 1;
		pos += 4;
		*(char *) &buffer[pos] = 0;
		pos += 4;
	}
	
	/*
	 * Encode the number of TaggedProfiles the IOR TaggedProfile sequence
	 * (1 or 0).
	 */
	*(unsigned int *) &buffer[pos] = (!!obj);
	pos += 4;
	
	if (obj) {
		unsigned int obj_key_len = obj->key._length;
		unsigned int host_name_len = strlen(obj->boa->orb->hostname);
		unsigned int profile_length
			= (  1 /* encapsulation (1 octet) */
			   + 2 /* IIOP::Version (2 chars) */
			   + 1 /* pad byte before host name length */
			   + 4 /* host name length */
			   + (host_name_len + 1) /* host name + NUL */
			   + ((host_name_len % 2) ? 0 : 1)
			     /* pad to get to 2-byte alignment */
			   + 2 /* port number */
			   + (((host_name_len % 4) >= 2) ? 2 : 0)
			     /* pad to get to 4-byte alignment */
			   + 4 /* object key length */
			   + obj_key_len /* object key length (incl. NUL) */
				);
		
		/* Encode the ProfileID and the length of the profile body. */
		*(unsigned int *) &buffer[pos] = IOP_TAG_INTERNET_IOP;
		pos += 4;
		*(unsigned int *) &buffer[pos] = profile_length;
		pos += 4;
		
		/* Enocde the encapsulation flag. */
		buffer[pos] = native_endianness;
		pos += 1;
		/* Encode the `IIOP::Version'. */
		buffer[pos] = 1;
		pos += 1;
		buffer[pos] = 0;
		pos += 1 + 1 /* pad byte */;
		
		/* Encode the host name. */
		*(unsigned int *) &buffer[pos] = host_name_len + 1;
		pos += 4;
		strncpy(&(buffer[pos]), obj->boa->orb->hostname,
			(host_name_len + 1));
		pos += (host_name_len + 1 /* NUL */ + 1 /* opt pad byte */)
		       & ~1;
		
		/* Encode the port number. */
		*(unsigned short *) &buffer[pos] = obj->boa->hostport;
		pos = (pos + 2 /* port */ + 3 /* opt pad */) & ~3;
		
		/* Encode the object key. */
		*(unsigned int *) &buffer[pos] = obj_key_len;
		pos += 4;
		memcpy(&(buffer[pos]), obj->key._buffer, obj_key_len);
		pos += obj_key_len;
	}
	buffer_len = pos;
	
	res = t_malloc(char, (buffer_len * 2) + 5);
	if (!res) {
		fprintf(stderr,
			"Error: can't malloc memory for stringified IOR.\n");
		return 0;
	}
	
	res[0] = 'I';
	res[1] = 'O';
	res[2] = 'R';
	res[3] = ':';
	for (i = 0; i < buffer_len; ++i)
		sprintf(&(res[i * 2 + 4]), "%02x", ((int) buffer[i]) & 0xff);
	res[(buffer_len * 2) + 4] = 0;
	
	return res;
}

static int hexbits(char a)
{
	if (a >= '0' && a <= '9')
		return a - '0';
	if ((a >= 'a' && a < 'g') ||
	    (a >= 'A' && a < 'G'))
		return (a & 15) + 9;
	return -1;
}

static int hexoctet(char x, char y)
{
	return hexbits(x) * 16 + hexbits(y);
}

/*
 * XXX --- The IIOP runtime should cache objects' IORs.
 *
 * XXX --- If there is a bug in this code, you should also check `flick_cdr_
 * decode_IOR_internal' to see if has the same bug.  Someday we should combine
 * the unmarshaling guts of these two functions.
 */
CORBA_Object CORBA_ORB_string_to_object(char *str)
{
	char *host_name;
	unsigned short boa_port;
	char *obj_type_id, *obj_key;
	
	struct hostent *he;
	CORBA_ORB orb;
	
	CORBA_Object res;
	unsigned int obj_type_id_len, obj_key_len, i;
	
	unsigned int endian_test = 1;
	char native_endianness = ((char *) &endian_test)[0];
	
	/*****/
	
	if (!str || !str[0]) {
		/* Null string object reference */
		fprintf(stderr,
			"Warning: empty object reference string.\n");
		return 0;
	}
	
	if (!strncmp("iiop:1.0//", str, 10)) {
		/* We're using a readable stringified object. */
		int host_end, port_end, type_end;
		
		for (host_end = 10;
		     str[host_end] && (str[host_end] != ':');
		     host_end++)
			/* No body. */
			;
		for (port_end = host_end;
		     str[port_end] && (str[port_end] != '/');
		     port_end++)
			/* No body. */
			;
		for (type_end = (str[port_end] ? (port_end + 1) : port_end);
		     str[type_end] && (str[type_end] != '/');
		     type_end++)
			/* No body. */
			;
		
		if (!str[host_end] || !str[port_end] || !str[type_end]) {
			fprintf(stderr,
				"Error: malformed URL-style object "
				"reference.\n");
			return 0;
		}
		
		str[host_end] = 0;
		str[port_end] = 0;
		str[type_end] = 0;
		
		host_name = &(str[10]);
		/*
		 * Convert `host_name' into a FQDN.  If we didn't do this, the
		 * IORs for `iiop:1.0/host/foo' and `iiop:1.0/host.dom/foo'
		 * would be different.  We want them to be the same so that our
		 * server doesn't think that the two IORs refer to objects in
		 * different ORBs.  (The server expects the hostnames to be
		 * identical; see `flick_cdr_build_obj' in the runtime header
		 * file <flick/encode/cdr.h>.)
		 */
		if (!(he = gethostbyname(host_name))) {
			fprintf(stderr,
				"Warning: unable to identify host `%s'.\n",
				host_name);
			host_name = t_malloc(char, strlen(&str[10]) + 1);
			if (!host_name) {
				fprintf(stderr,
					"Error: can't malloc memory for host "
					"name.\n");
				return 0;
			}
			strcpy(host_name, &str[10]);
		} else {
			host_name = t_malloc(char, strlen(he->h_name) + 1);
			if (!host_name) {
				fprintf(stderr,
					"Error: can't malloc memory for host "
					"name.\n");
				return 0;
			}
			strcpy(host_name, he->h_name);
		}
		
		boa_port = atoi(&str[host_end + 1]);
		
		obj_type_id_len = strlen(&str[port_end + 1]);
		obj_type_id = t_malloc(char, (obj_type_id_len + 1));
		if (!obj_type_id) {
			fprintf(stderr,
				"Error: can't malloc memory for object "
				"type.\n");
			free(host_name);
			return 0;
		}
		strncpy(obj_type_id,
			&str[port_end + 1],
			(obj_type_id_len + 1));
		
		obj_key_len = strlen(&str[type_end + 1]);
		obj_key = t_malloc(char, obj_key_len);
		if (!obj_key) {
			fprintf(stderr,
				"Error: can't malloc memory for object "
				"key.\n");
			free(obj_type_id);
			free(host_name);
			return 0;
		}
		memcpy(obj_key, &str[type_end + 1], obj_key_len);
		
	} else if (!strncmp("IOR:", str, 4)) {
		/* We're using the CORBA 2.0 p10.17 stringified object. */
		
		char buf[1024]; /* XXX --- should dynamically alloc `buf'. */
		unsigned int pos, len;
		
		char encapsulation_endianness;
		unsigned int profile_count;
		
		/* Skip over the ``IOR:'' prefix. */
		str += 4;
		
		/*
		 * Because every byte in the IOR is represented by two hex
		 * digits in the string, the string must contain an even number
		 * of characters.
		 */
		len = strlen(str);
		if (len % 2) {
			fprintf(stderr,
				"Error: invalid IOR (odd number of hex "
				"digits).\n");
			return 0;
		}
		if ((len / 2) > sizeof(buf)) {
			/* XXX */
			fprintf(stderr,
				"Error: IOR too long.\n");
			return 0;
		}
		
		/* Decode the hex string into a sequence of bytes in `buf'. */
		for (i = 0; i < len; i += 2) {
			int val = hexoctet(str[i], str[i + 1]);
			if (val < 0) {
				fprintf(stderr,
					"Error: invalid IOR (contains non-hex "
					"digits).\n");
				return 0;
			} else
				buf[i / 2] = val;
		}
		
		/*
		 * `buf' now holds a CORBA encapsulation of an IOR as described
		 * in Section 10.6.2 of the CORBA 2.0 specification.
		 *
		 * `pos' is our index into `buf'.
		 */
		pos = 0;
		encapsulation_endianness = buf[pos];
		pos += 4;
		
		/*
		 * First, get the object `type_id', a string.
		 */
		if (encapsulation_endianness == native_endianness)
			len = (*(unsigned int *) &buf[pos]);
		else
			len = swap_long(*(unsigned int *) &buf[pos]);
		pos += 4;
		
		obj_type_id_len = len - 1;
		obj_type_id = t_malloc(char, len);
		if (!obj_type_id) {
			fprintf(stderr,
				"Error: can't malloc memory for object "
				"type_id.\n");
			return 0;
		}
		strncpy(obj_type_id, &(buf[pos]), len);
		if (obj_type_id[len - 1] != 0) {
			/* XXX --- This should never happen. */
			fprintf(stderr,
				"Warning: IOR object type_id was not NUL "
				"terminated.  Terminating it.\n");
			obj_type_id[len - 1] = 0;
		}
		pos += (len + 3) & ~3;
		
		/*
		 * Now, find the IOP Profile within our IOR's sequence of
		 * TaggedProfiles.
		 */
		if (encapsulation_endianness == native_endianness)
			profile_count = (*(unsigned int *) &buf[pos]);
		else
			profile_count = swap_long(*(unsigned int *) &buf[pos]);
		pos += 4;
		if (profile_count == 0) {
			/*
			 * No profiles?  Then this is a null object reference
			 * (Section 10.6.2).
			 */
			free(obj_type_id);
			return 0;
		}
		
		for (i = 0; i < profile_count; ++i) {
			IOP_ProfileId tag;
			
			if (encapsulation_endianness == native_endianness) {
				tag = (*(unsigned int *) &buf[pos]);
				pos += 4;
				len = (*(unsigned int *) &buf[pos]);
				pos += 4;
			} else {
				tag = swap_long(*(unsigned int *) &buf[pos]);
				pos += 4;
				len = swap_long(*(unsigned int *) &buf[pos]);
				pos += 4;
			}
			
			if (tag == IOP_TAG_INTERNET_IOP) {
				/*
				 * Is this an IIOP 1.0 Profile?  Note that
				 * `buf[pos]' currently points to an
				 * encapsulation header (a char).
				 */
				if ((buf[pos + 1] == 1) && buf[pos + 2] == 0)
					break;
			}
			/* Skip over this profile. */
			pos += (len + 3) & ~3;
		}
		if (i >= profile_count) {
			fprintf(stderr,
				"Warning: no IIOP Profile in IOR.  Returning "
				"a null object reference.\n");
			free(obj_type_id);
			return 0;
		}
		
		/*
		 * At this point, `pos' is the index of the IIOP Profile
		 * encapsulation in `buf'.  `pos' is also 4-byte aligned, since
		 * we just read past the (unsigned long) ProfileId.
		 *
		 * By adding 4 to `pos' here, we skip over the endianness
		 * indicator (1 octet) for the IIOP Profile encapsulation, the
		 * `IIOP::Version' struct (2 chars), and the pad byte preceding
		 * the `host' string length.
		 */
		encapsulation_endianness = buf[pos];
		pos += 4;
		
		/* Get the host name. */
		if (encapsulation_endianness == native_endianness)
			len = (*(unsigned int *) &buf[pos]);
		else
			len = swap_long(*(unsigned int *) &buf[pos]);
		pos += 4;
		
		host_name = t_malloc(char, len);
		if (!host_name) {
			fprintf(stderr,
				"Error: can't malloc memory for host name.\n");
			free(obj_type_id);
			return 0;
		}
		strncpy(host_name, &(buf[pos]), len);
		if (host_name[len - 1] != 0) {
			/* XXX --- This should never happen. */
			fprintf(stderr,
				"Warning: IIOP host name was not NUL "
				"terminated.  Terminating it.\n");
			host_name[len - 1] = 0;
		}
		pos += (len + 1) & ~1;
			
		/* Get the BOA port. */
		if (encapsulation_endianness == native_endianness)
			boa_port = (*(unsigned short *) &buf[pos]);
		else
			boa_port = swap_short(*(unsigned short *) &buf[pos]);
		pos += 2;
		
		/* Get the object key. */
		/* Align for object key length. */
		pos = (pos + 3) & ~3;
		if (encapsulation_endianness == native_endianness)
			obj_key_len = (*(unsigned long *) &buf[pos]);
		else
			obj_key_len = swap_long(*(unsigned long *) &buf[pos]);
		pos += 4;
		
		obj_key = t_malloc(char, obj_key_len);
		if (!obj_key) {
			fprintf(stderr,
				"Error: can't malloc memory for object "
				"key.\n");
			free(host_name);
			free(obj_type_id);
			return 0;
		}
		memcpy(obj_key, &(buf[pos]), obj_key_len);
		/*
		 * Note that the object key is a sequence of octets, *not* a
		 * NUL-terminated string.  So do not check for termination.
		 */
		pos += obj_key_len; /* Leave `pos' at single-byte alignment. */
		
		/*
		 * Convert the `host_name' into a FQDN, which we use to key our
		 * ORBs.
		 */
		if (!(he = gethostbyname(host_name))) {
			fprintf(stderr,
				"Warning: unable to identify host `%s'.\n",
				host_name);
		} else {
			free(host_name);
			host_name = t_malloc(char, strlen(he->h_name) + 1);
			if (!host_name) {
				fprintf(stderr,
					"Error: can't malloc memory for host "
					"name.\n");
				free(obj_type_id);
				free(obj_key);
				return 0;
			}
			strcpy(host_name, he->h_name);
		}
		
	} else {
		/* I have no idea what kind of object reference this is. */
		fprintf(stderr,
			"Error: unrecognized object reference format.\n");
		return 0;
	}
	
	/*
	 * At this point we have `host_name', `boa_port', `obj_type_id', and
	 * `obj_key' set.  We need to build the object --- but don't try to
	 * connect to it just yet.
	 */
	for (i = 0; i < (unsigned int) orb_count; ++i)
		if (!strcmp(orbs[i]->hostname, host_name))
			break;
	
	if (((int) i) == orb_count) {
		CORBA_ORB *new_orbs;
		
		new_orbs = t_realloc(orbs, CORBA_ORB, (orb_count + 1));
		if (!new_orbs) {
			fprintf(stderr,
				"Error: can't malloc memory for new ORB.\n");
			free(host_name);
			free(obj_type_id);
			free(obj_key);
			return 0;
		}
		orbs = new_orbs;
		++orb_count;
		
		orbs[i] = t_calloc(struct CORBA_ORB_type, 1);
		if (!orbs[i]) {
			fprintf(stderr,
				"Error: can't malloc memory for new ORB.\n");
			free(host_name);
			free(obj_type_id);
			free(obj_key);
			return 0;
		}
		
		orbs[i]->hostname = host_name;
		
	} else
		/*
		 * Don't need `host_name' anymore; the located ORB already has
		 * a copy of its name.
		 */
		free(host_name);
	
	orb = orbs[i];
	
	for (i = 0; (int)i < orb->OA_count; ++i)
		if (orb->boas[i]->hostport == boa_port)
			break;
	
	if (((int) i) == orb->OA_count) {
		CORBA_BOA *new_boas;
		
		new_boas = t_realloc(orb->boas,
				     CORBA_BOA, (orb->OA_count + 1));
		if (!new_boas) {
			fprintf(stderr,
				"Error: can't malloc memory for new BOA.\n");
			free(obj_type_id);
			free(obj_key);
			return 0;
		}
		
		new_boas[i] = t_calloc(struct CORBA_BOA_type, 1);
		if (!new_boas[i]) {
			fprintf(stderr,
				"Error: can't malloc memory for new BOA.\n");
			free(obj_type_id);
			free(obj_key);
			return 0;
		}
		
		orb->boas = new_boas;
		orb->OA_count += 1;
		
		orb->boas[i]->orb = orb;
		orb->boas[i]->hostport = boa_port;
	}
	
	/*
	 * Finally, we can create the CORBA_Object!  Yippee!
	 */
	res = t_malloc(struct flick_client, 1);
	if (!res) {
		fprintf(stderr,
			"Error: can't malloc memory for CORBA_Object.\n");
		free(obj_type_id);
		free(obj_key);
		return 0;
	}
	
	res->boa = orb->boas[i];
	res->key._buffer = obj_key;
	res->key._length = obj_key_len;
	res->key._maximum = 1024;
	res->type_id = obj_type_id;
	res->type_id_len = obj_type_id_len;
	
	return res;
}

/* End of file. */

