/*
 * Copyright (c) 1997 The University of Utah and
 * the Computer Systems Laboratory at the University of Utah (CSL).
 *
 * This file is part of Flick, the Flexible IDL Compiler Kit.
 *
 * Flick is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Flick is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Flick; see the file COPYING.  If not, write to
 * the Free Software Foundation, 59 Temple Place #330, Boston, MA 02111, USA.
 */

#include <stdio.h>
#include <unistd.h>
#include <sys/types.h>
#include <mach_init.h>
#include <mach/message.h>
#include <mach/port.h>
#include <mach/mach_port.h>
#include <mach/mig_errors.h>
#include <memory.h>
#include <servers/netname.h>

/* defined in "-work" server program */
boolean_t call_server(mach_msg_header_t *request_ptr,
                      mach_msg_header_t *reply_ptr);


#define MSGBUFSIZE (65*1024)

void *__buffer = 0;
int   __buflen = 0;


/*
 * Support functions
 */

void
die(char *mes, int code)
{
	printf("fatal error: %s (%d, 0x%08x)\n", mes, code, code);
/*	printf("About to seg fault [weird - exit(1) causes a seg fault].\n");*/
	exit(1);
	printf("We haven't died!\n");
}

#define diemach(func, params) \
({ int err = func params; if (err) die (#func, err); })

__inline mach_msg_header_t *
dispatch(mach_msg_header_t *request_ptr, mach_msg_header_t *reply_ptr)
{
	if (!call_server(request_ptr, reply_ptr)) {
		printf("%08x %08x %d\n",
		       request_ptr->msgh_bits,
		       request_ptr->msgh_size,
		       request_ptr->msgh_id);
		printf("Invalid message (id = %d) not deallocated.\n",
		       request_ptr->msgh_id);
	}
	return reply_ptr;       
}

typedef union {
	mach_msg_header_t	hdr;
	mig_reply_header_t	death_pill;
	char			space[MSGBUFSIZE];
} reply_message;

int
main(int argc, char *argv[])
{
	mach_port_t task_self = mach_task_self();
	mach_port_t right;
	reply_message msg_buffer_1, msg_buffer_2;

	mach_msg_header_t * request_ptr;
	mig_reply_header_t * reply_ptr;

	int ret;
      
	if (argc <=1 ) {
		fprintf(stderr, "usage: %s server_name\n", argv[0]);
		return 1;
	}

	/* Create a receive right to receive requests from the client */
	diemach(mach_port_allocate,(task_self, MACH_PORT_RIGHT_RECEIVE, &right));

	request_ptr = &msg_buffer_1.hdr;
	reply_ptr = &msg_buffer_2.death_pill;
	printf("server buffers at %08x-%08x and %08x-%08x\n",
	       (int)request_ptr, (int)request_ptr+MSGBUFSIZE,
	       (int)reply_ptr, (int)reply_ptr+MSGBUFSIZE);

	/* Advertise our server name so clients can connect. */
	printf("Setting up service as: %s\n", argv[1]);
	diemach(netname_check_in,(name_server_port, argv[1], task_self, right));

	/* use mach_msg_server as our server handling loop */
	if ((ret = mach_msg_server(dispatch, MSGBUFSIZE, right,
				   MACH_MSG_OPTION_NONE)) != KERN_SUCCESS) {
		die ("mach_msg_server loop", ret);
	}

	printf ("\nparent terminating\n");

	/* unregister with the name server */
	diemach(netname_check_out,(name_server_port, argv[1], task_self));
	return 0;
}

/* End of file. */
